% *************************************************************
% GABSOBHIN - Genetic Algorithms Based Synthesis & Optimization
% of Batch Heat Integration Networks - Copyright LENI-EPFL 2000
% Development Pierre KRUMMENACHER & Blaise RENAUD for LENI-EPFL 
% *************************************************************
% I_Const_Def.m /  08.10.00 - .........
% *************************************************************
% Definition of the general constants of the program, including
% the units of data defined as a structure of strings
% *************************************************************


%	Print control:

if PSOS.help_print == 1
   help I_Const_Def
end

if PSOS.step_print == 1
   disp('*** I_Const_Def ***');
end



%	Definition of general constants:

Cte.Tmax = 10000;
Cte.Tmin = -10000;
Cte.Tk_init = 0;
Cte.one_third = 1/3;
Cte.pausetext = 'Press any key ...';
Cte.HSU_DT_min = 1;



%	Maximum number of lines of tables (related to the size of reserved tables in IHRS_Data_Input (Excel Workbook)):

Cte.RIpmax = 100;
Cte.RJpmax = 100;
Cte.RtPD_Ipmax = 300; %   RtPD_Ipmax can be set independently of RIpmax  
Cte.RtPD_Jpmax = 300; %   RtPD_Jpmax can be set independently of RJpmax
Cte.NtPDmax_per_line = 10; %   maximum number of time periods per line
Cte.RMpmax = 10;
Cte.RNpmax = 10;
Cte.Omax = 10;
Cte.RXTmax = 20;
Cte.RSTmax = 10;
Cte.Kmax = 10;



%   Name of mex files to be used "around" optimize.dll:

Cte.OF_mexfile_name = 'I_OF'; %   name of mex file to be called by optimize for OF calculation
Cte.Report_Evol_mexfile_name = 'I_Report_Evolution';    %   name of mex file to be called by optimize to report evolution
Cte.Distance_name = 'Variable_weights';	%    name of distance function to be used by GA to calculate the distance between two
%   individuals for replacement; available are 'Euclidian', 'Temperature', 'Variable_weights' and 'my_comparator'


%	Corresponding row and column offset for the MW HEX specification matrix:

ID.roff_CPS = 0;
ID.roff_HUS = Cte.RJpmax;
ID.roff_CUS = Cte.RJpmax + Cte.RMpmax;
ID.roff_HSF = Cte.RJpmax + Cte.RMpmax + Cte.RNpmax;
ID.coff_HPS = 0;
ID.coff_CUS = Cte.RIpmax;
ID.coff_HUS = Cte.RIpmax + Cte.RNpmax;
ID.coff_HSF = Cte.RIpmax + Cte.RNpmax + Cte.RMpmax;

%   other ID offset values are defined in I_Data_Input while reading indput data

%	Strings defining the units of data:

%DU.money_unit = 'CHF';	%	default setting for economic unit, could be replaced later while reading input file, or directly from the GUI
DU.money_unit = 'DKK';	%	default setting for economic unit, could be replaced later while reading input file, or directly from the GUI

DU.time_sec = 's';
DU.time_min = 'min';
DU.time_hr = 'h';
DU.time_year = 'year';

DU.temperature = 'C';


DU.mass = 'kg';
DU.mass_B = 'kg/batch';
DU.mass_flowrate = 'kg/s';


DU.density = 'kg/m3';
DU.specific_heat = 'kJ/(kg*C)';
DU.heat_transfer = 'kW/(m2*C)';


DU.heat_flowrate = 'kW/C';
DU.heat_capac = 'kWh/C';
DU.heat_rate = 'kW';

DU.heat_kWh = 'kWh';
DU.heat_kJ = 'kJ';

DU.heat_kWh_A = 'kWh/year';

DU.heat_kWh_B = 'kWh/batch';
DU.heat_kJ_B = 'kJ/batch';

DU.area = 'm2';
DU.capacity = 'm3';


DU.cost = DU.money_unit;
DU.specific_cost = [DU.money_unit,'/m3'];
DU.specific_energy_cost = [DU.money_unit,'/kWh'];
DU.cost_A = [DU.money_unit,'/year'];
DU.cost_B = [DU.money_unit,'/batch'];


DU.no_units = '-';

DU.HR_axis_label = 'Heat recovery [kWh/batch]';
DU.TBCs_axis_label = ['TBCs [',DU.cost_B,']'];
DU.Gen_axis_label = 'Generation [-]';


% *************************************************************
