% *************************************************************
% GABSOBHIN - Genetic Algorithms Based Synthesis & Optimization
% of Batch Heat Integration Networks - Copyright LENI-EPFL 2000
% Development Pierre KRUMMENACHER & Blaise RENAUD for LENI-EPFL 
% *************************************************************
% I_Exec_DLL.m /  01.02.02 - .........
% *************************************************************
% Run file for optimize.dll mex file (execute dll)
% *************************************************************
% INPUT:
%		:	
%
% OUTPUT:
%		:	
% *************************************************************
% to do:
% INTERNAL VARIABLES
%		:	
% *************************************************************


if PSOS.help_print == 1
   help I_Exec_DLL
end

if PSOS.step_print == 1
   disp('*** I_Exec_DLL ***');
end


standalone = 0;

if standalone == 0

%	DOS command to rename possibly already existing file to avoid overwritting
%	! move optimize_1.log optimize_1prev.log				% Does not work at OW

diary optimize_1.log

disp ('@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@');
disp ('Starting I_Exec_DLL.m Matlab procedure for optimization with Struggle');
disp ('@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@');
disp ( ['          ',datestr(now)] );
disp ('@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@');
disp ('');


% *************************************************************

%	vector specifying which variables are fixed (0 for binary, 0.1 for real) and which are to be optimized (1 for binary, 0.9 for real)
%	IV.X_opt (read and checked for consistency in I_Data_Input, possibly overwritten in I_Var_Init if PSOS.FOIV = 1)

%	vector specifying the value of variables which are fixed (depending on the value of the corresponding variable of IV.X_opt)
%	IV.X_val (read and checked for consistency in I_Data_Input)

%	vector of lower bound values for each variable to be generated by the GA
%	IV.X_min generated in I_Var_Init

%	vector of upper bound values for each variable to be generated by the GA
%	IV.X_max generated in I_Var_Init

%	vector of weight factors to be used for the default cartesian distance measurement (not used here)
%	IV.X_wei generated in I_Var_Init


%	!!! the management of initial population and seeding individual remains to be done (should be read in I_Input_Data)

% structure that defines general user data
UserData.inXsize = PSOS.NIV;
UserData.inXopt = IV.X_opt;
UserData.inXval = IV.X_val;
UserData.inXmin = IV.X_min;
UserData.inXmax = IV.X_max;
UserData.inXwei = IV.X_wei;

UserData.inDsize = DD.NDV;
UserData.inDwei = DD.D_wei;
UserData.Objective = Cte.OF_mexfile_name;    %   name of mex file to be called by optimize for OF calculation (set in I_Const_Def)

% structure that defines the parameters
Parameters.inN_POP = PSOS.NPop;
Parameters.inN_GEN = PSOS.NGen;
Parameters.inP_CRO = PSOS.PCroRIV;
Parameters.BCORdep = PSOS.EFRCro;
Parameters.inP_MUT = PSOS.PMutRIV;
Parameters.BCOIdep = PSOS.EFICro;
Parameters.BCOname = PSOS.COBIV;
Parameters.BCOBmut = PSOS.PMutBIV;

% structure that defines the control data
Control.PnPlot = PSOS.f_plot;
Control.PTrace = PSOS.O_trace_print;
Control.Comparator = Cte.Distance_name;    %    name of distance function to be used (set in I_Const_Def)
Control.Panel = Cte.Report_Evol_mexfile_name;   %   name of mex file to be called by optimize to report evolution (set in I_Const_Def)

% structure that defines the population data
OptPop.PopNum = PSOS.NPop;
OptPop.VarNum = PSOS.NIV;
OptPop.OptPop = -9.87654321 * ones(PSOS.NIV,PSOS.NPop);


% *************************************************************

%	Definition of how many values (functions) are to be returned from I_OF to optimize.dll by the yret1 vector
%	after each genome evaluation. These values are then registered with each genome using the associated
%	EvalData structure. The size is OF_side is up to the user, but the following rules should be observed:
%	1) decide which values are to be returned and defined using the OF_side_name matrix below (documentation).
%	2) make sure that returned values by yret1 at the end of I_OF are consistent with that defined by the 
%		matrix name.
%	3)	for historical reasons, the role of the 4th, 5th and 6th elements of the vector is "hard coded" in
%		optimize.dll, since these are sent without possible parametrization to I_Report_Evolution as:
%		4th elem. = the best objective function (1st input param of I_Report_Evolution designated as "OF_best"	)
%		5th elem. = the mean value (over the population of the current generation) of the objective function
%						(2nd input param of I_Report_Evolution designated as "OF_mean"	)
%		6th elem. = the side value of the objective function (3rd input param of I_Report_Evolution designated,
%						for the particular problem at hand, as "Side_Val_of_OF_best")
%	Note that whatever the value returned as the 5th element, the mean value of the OF over the current
%	population is calculated by GAmain, as this data cannot be calculated at the individual level of one
%	individual in I_OF.
%	Similarly, the value returned to I_Report_Evolution as 1st input param "OF_best" is based on the value returned from
%	I_OF as the 4th element of yret1 (OF value), but GAmain manages the search for the best OF over the
%	population of the current generation. Having identified the best genome of the population, the 3rd input
%	param returned to I_Report_Evolution as "Side_Val_of_OF_best" is taken in read in th 6th element of the 
%	EvalData of the best genome.
%	In conclusion, the OF_best, OF_mean, Side_Val_of_OF_best are not directly read from the 4th , 5th and 6th
%	element of yret1 of I_OF, but are nevertheless in strong link. 

OF_side_size = 15;

OF_side = zeros(1,OF_side_size);

OF_side_name1(1,:) 	= 'Total batch costs          [CHF/batch] ';
OF_side_name1(2,:) 	= 'Actual heat recovery       [kWh/batch] ';
OF_side_name1(3,:) 	= 'Oper. costs of HU and CU   [CHF/batch] ';
OF_side_name1(4,:) 	= 'Best OF of current population     [..] ';
OF_side_name1(5,:) 	= 'Average OF of current population  [..] ';
OF_side_name1(6,:) 	= 'Side value (HR) of best OF        [..] ';
OF_side_name1(7,:) 	= 'Capital costs of HEXs      [CHF/batch] ';
OF_side_name1(8,:) 	= 'Capital costs of HSUs      [CHF/batch] ';
OF_side_name1(9,:) 	= 'Capi. costs of T control   [CHF/batch] ';
OF_side_name1(10,:) 	= 'Capi. costs of hot utility [CHF/batch] ';
OF_side_name1(11,:) 	= 'Capi. costs of cold util.  [CHF/batch] ';
OF_side_name1(12,:)	= 'Oper. costs of hot utility [CHF/batch] ';
OF_side_name1(13,:) 	= 'Oper. costs of cold util.  [CHF/batch] ';
OF_side_name1(14,:) 	= 'Total hot utility          [kWh/batch] ';
OF_side_name1(15,:) 	= 'Total cold utility         [kWh/batch] ';


OF_side_name2(1,:) 	= 'TBCs              ';
OF_side_name2(2,:) 	= 'HR_of_TBCs        ';
OF_side_name2(3,:) 	= 'HU+CU oper. costs ';
OF_side_name2(4,:) 	= 'Best OF of popul. ';
OF_side_name2(5,:) 	= 'Aver. OF of popul.';
OF_side_name2(6,:) 	= 'HR of best OF     ';
OF_side_name2(7,:) 	= 'HEXs B costs      ';
OF_side_name2(8,:) 	= 'HSUs B costs      ';
OF_side_name2(9,:) 	= 'TC B costs        ';
OF_side_name2(10,:) 	= 'HU capi. B costs  ';
OF_side_name2(11,:) 	= 'CU capi. B costs  ';
OF_side_name2(12,:)	= 'HU oper. costs    ';
OF_side_name2(13,:) 	= 'CU oper. costs    ';
OF_side_name2(14,:) 	= 'HU tot            ';
OF_side_name2(15,:) 	= 'CU tot            ';

%	See yret1 assignments in the I_Return_OF_Value module of I_OF
%yret1(1) = RD.TBCs;
%yret1(2) = RD.HR;
%yret1(3) = RD.HU_oper_Costs + RD.CU_oper_Costs;
%yret1(4) = OF;
%yret1(5) = 0;	%	to be used to store OF_mean (managed by C++ optimize/GAmain side)
%yret1(6) = Side_Val;
%yret1(7) = RD.HEX_B_Costs;
%yret1(8) = RD.HSU_B_Costs;
%yret1(9) = RD.TC_B_Costs;
%yret1(10) = RD.HU_capi_B_Costs;
%yret1(11) = RD.CU_capi_B_Costs;
%yret1(12) = RD.HU_oper_Costs;
%yret1(13) = RD.CU_oper_Costs;
%yret1(14) = RD.HU_tot;
%yret1(15) = RD.CU_tot;



tic;

%	calling for optimize.dll:




[BestGenome, OF_side, N_OF_side, OptPopOut] = optimize ( UserData,Parameters,Control,OptPop );	% call DLL 

%	Note that BestGenome has the same structure as UserData (see above), i.e. it has the fields:
%	....inXsize .inXopt .inXval .inXmin .inXmax .inXwei .inDsize and .inDwei, for the best genome of final
%	population. But of these, the only field of interest if .inXval, which is the optimum vector of
%	independent variables (which possibly includes the values of independent variables that have been fixed
%	as specified by the inXopt vector).
%
%	Similar reflections can be made for OptPopOut and OptPop...
%	OptPop in created, the values of OptPop.OptPop are of no interest.
%	OptPop.PopNum and OptPop.VarNum will be used



disp(' Back after call to optimize.dll. Return to I_Exec_DLL.m !!!!! ');
disp(' ');
dispnum ( N_OF_side,  'Number of side functions  ', '  N_OF_side' );
for i=1:N_OF_side;
	dispnum ( OF_side(i), OF_side_name1(i,:), ['  OF_side(',num2str(i),') = ', OF_side_name2(i,:)] );
end;
disp(' ');
disparray ( BestGenome.inXval, BestGenome.inXsize, 5, 'BestGenome' );
disp(' ');


%disparray(OptPopOut.OptPop,OptPopOut.VarNum*OptPopOut.PopNum,OptPopOut.PopNum,'OptPop');
%pause
%disp(' ************* ');
%disparray(OptPopOut.OptPop',OptPopOut.VarNum*OptPopOut.PopNum,OptPopOut.VarNum,'OptPop');
%pause
%disp(' ************* ');
	%disp(' ');
%for ii=1:OptPopOut.PopNum
	%disparray(OptPopOut.OptPop(:,ii)',312,7,['OptPop(',num2str(ii),')']);
	%disp(' ');
%end
%disp(' ************* ');
%pause

telapsed=toc;
dispnum ( telapsed, 'time elapsed in I_Exec_DLL.m', 'sec' );
disp(' ********************************************************************************************** ');
diary off			%	to close the log file
clear optimize		%	to set the handle of optimize.dll file free
save optimize


% ========================================================================

else	%	i.e. standalone ~=0


%	Data for stand-alone I_OF execution:

X_bHPS = [1,1,1,1];

X_bCPS = [1,1,1];

%X_rHSU = [0.065594, 0.39634, 0.15233, 0.43535];

%X_rHSU = [0.01, 0.7, 0.6, 0.2];

%X_rHSU = [0.05, 0.05, 0.7, 0.2];

X_rHSU = rand(1,4);

%X_rHSU = (0.5827916815612309,    0.4234962568510508,    0.5155117521407628,   0.3339514799717586);

%X_rHSU = [0.4450964322879468, 0.9318145784616647, 0.4659943416754240, 0.4186494677275062];

%X_rHPS = [0.71584, 0.61591, 0.80685, 1, 0.92498, 0.78181, 0.67917, 1, 0.84893, 0.98262, 0.46062, 1, 0.33335, 0.99412, 0.46479, 1];

%X_rHPS = [0.71584, 0.61591, 0.5, 1, 0.92498, 0.78181, 0.67917, 1, 0.84893, 0.98262, 0.46062, 1, 0.33335, 0.99412, 0.46479, 1];

X_rHPS = rand(1,16);

%X_rCPS = [0.87201, 0.73204, 0.94804, 1, 0.69632, 0.17481, 0.76182, 1, 0.76795, 0.81465, 0.98279 1];

X_rCPS = rand(1,12);

X_iv = [X_bHPS X_bCPS X_rHSU X_rHPS X_rCPS];

X_iv_Size = size(X_iv);


end
